@@ -13,9 +13,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from tempfile import mkstemp
+
 from oslo_log import log
 import six
 from six.moves.urllib.request import urlopen
+from tempest.common import waiters
+from tempest import config
+from tempest.lib.common.utils import data_utils
+from tempest.lib.common.utils import test_utils
+from tempest.lib import exceptions
 
 from manila_tempest_tests.common import constants
 from manila_tempest_tests.common import remote_client
@@ -23,13 +30,6 @@ from manila_tempest_tests.tests.api import base
 from manila_tempest_tests.tests.scenario import manager
 from manila_tempest_tests import utils
 
-from tempest.common import waiters
-from tempest import config
-from tempest.lib.common.utils import data_utils
-from tempest.lib import exceptions
-
-from tempfile import mkstemp
-
 CONF = config.CONF
 LOG = log.getLogger(__name__)
 
@@ -91,24 +91,13 @@ class ShareScenarioTest(manager.NetworkScenarioTest):
                 msg = ("Image %s not found. Expecting an image including "
                        "required share tools." %
                        CONF.share.image_with_share_tools)
-                raise exceptions.InvalidConfiguration(message=msg)
+                raise exceptions.InvalidConfiguration(msg)
         self.ssh_user = CONF.share.image_username
         LOG.debug('Starting test for i:{image_id}, f:{flavor}. '
                   'user: {ssh_user}'.format(image_id=self.image_id,
                                             flavor=self.flavor_ref,
                                             ssh_user=self.ssh_user))
 
-        self.security_group = self._create_security_group()
-        self.network = self._create_network(namestart="manila-share")
-        self.subnet = self._create_subnet(
-            network=self.network,
-            namestart="manila-share-sub",
-            ip_version=self.ip_version,
-            use_default_subnetpool=self.ipv6_enabled)
-        router = self._get_router()
-        self._create_router_interface(subnet_id=self.subnet['id'],
-                                      router_id=router['id'])
-
         self.storage_network = (
             self._get_network_by_name_or_id(CONF.share.storage_network)
             if CONF.share.storage_network else None
@@ -117,6 +106,24 @@ class ShareScenarioTest(manager.NetworkScenarioTest):
             self.storage_network['name'] if self.storage_network else None
         )
 
+        # Tests need to be able to ssh into the VM - so we need
+        # a security group, and a tenant private network
+        self.security_group = self._create_security_group()
+        self.network = self._create_network(namestart="manila-share")
+        # When not using a "storage network" to connect shares to VMs,
+        # we need the subnet to match the IP version we're testing
+        subnet_ip_params = {} if self.storage_network else {
+            'ip_version': self.ip_version,
+            'use_default_subnetpool': self.ipv6_enabled
+        }
+        self.subnet = self._create_subnet(
+            network=self.network,
+            namestart="manila-share-sub",
+            **subnet_ip_params)
+        router = self._get_router()
+        self._create_router_interface(subnet_id=self.subnet['id'],
+                                      router_id=router['id'])
+
         if CONF.share.multitenancy_enabled:
             # Skip if DHSS=False
             self.share_network = self.create_share_network()
@@ -193,6 +200,24 @@ class ShareScenarioTest(manager.NetworkScenarioTest):
         self.share = self.shares_client.get_share(self.share['id'])
         return remote_client
 
+    def validate_ping_to_export_location(self, export, remote_client,
+                                         ping_timeout=None):
+        timeout = ping_timeout or CONF.validation.ping_timeout
+
+        def ping_to_export_location(export, remote_client):
+            ip, version = self.get_ip_and_version_from_export_location(export)
+            try:
+                remote_client.exec_command(
+                    "ping{} -c5 -w1 {}".format(
+                        '6' if version == 6 else '', ip))
+                return True
+            except exceptions.SSHExecCommandFailed:
+                return False
+
+        test_utils.call_until_true(ping_to_export_location,
+                                   timeout, 1, export=export,
+                                   remote_client=remote_client)
+
     def write_data_to_mounted_share(self, escaped_string, remote_client,
                                     mount_point='/mnt/t1'):
         remote_client.exec_command("echo \"{escaped_string}\" "
@@ -351,41 +376,62 @@ class ShareScenarioTest(manager.NetworkScenarioTest):
                 snapshot=snapshot, access_level=access_level, client=client)
 
     def provide_access_to_client_identified_by_cephx(self, share=None,
+                                                     access_rule=None,
                                                      access_level='rw',
                                                      access_to=None,
                                                      remote_client=None,
                                                      locations=None,
                                                      client=None,
                                                      oc_size=20971520):
-        share = share or self.share
-        client = client or self.shares_v2_client
-        access_to = access_to or data_utils.rand_name(
-            self.__class__.__name__ + '-cephx-id')
-        # Check if access is already granted to the client
-        access = self.shares_v2_client.list_access_rules(
-            share['id'], metadata={'metadata': {'access_to': access_to}})
-        access = access[0] if access else None
+        """Provide an access to a client identified by cephx authentication
 
-        if not access:
-            access = self._allow_access(
-                share['id'], access_level=access_level, access_to=access_to,
-                access_type="cephx", cleanup=False, client=client)
-            # Set metadata to access rule to be filtered if necessary.
-            # This is necessary to prevent granting access to a client who
-            # already has.
-            self.shares_v2_client.update_access_metadata(
-                metadata={"access_to": "{}".format(access_to)},
-                access_id=access['id'])
-        get_access = self.shares_v2_client.get_access(access['id'])
+        :param: share: An existing share.
+        :param: access_rule: An existing access rule. In case we want to create
+                        the configuration files in the instance according to an
+                        existing access rule.
+        :param: access_level: Share access level; this is not required if
+                        "access_rule" is set.
+        :param: access_to: Client to provide access to; this is not required if
+                        "access_rule" is set.
+        :param: remote_client: An SSH client connection to the Nova instance.
+        :param: locations: Export locations of shares.
+        :param: client: Client object.
+        :param: oc_size: Set how many bytes of data will the client cache.
+        :return: Share access.
+
+        """
+        client = client or self.shares_v2_client
+        if not access_rule:
+            share = share or self.share
+            access_to = access_to or data_utils.rand_name(
+                self.__class__.__name__ + '-cephx-id')
+            # Check if access is already granted to the client
+            access_rules_matching_client = client.list_access_rules(
+                share['id'], metadata={'metadata': {'access_to': access_to}})
+            access_rule = (access_rules_matching_client[0] if
+                           access_rules_matching_client else None)
+
+            if not access_rule:
+                access_rule = self._allow_access(
+                    share['id'], access_level=access_level,
+                    access_to=access_to, access_type="cephx", cleanup=False,
+                    client=client)
+                # Set metadata to access rule to be filtered if necessary.
+                # This is necessary to prevent granting access to a client who
+                # already has.
+                client.update_access_metadata(
+                    metadata={"access_to": "{}".format(access_to)},
+                    access_id=access_rule['id'])
+        get_access = client.get_access(access_rule['id'])
         # Set 'access_key' and 'access_to' attributes for being use in mount
         # operation.
         setattr(self, 'access_key', get_access['access_key'])
-        setattr(self, 'access_to', access_to)
+        setattr(self, 'access_to', get_access['access_to'])
 
         remote_client.exec_command(
             "sudo crudini --set {access_to}.keyring client.{access_to} key "
             "{access_key}"
-            .format(access_to=access_to, access_key=self.access_key))
+            .format(access_to=self.access_to, access_key=self.access_key))
         remote_client.exec_command(
             "sudo crudini --set ceph.conf client \"client quota\" true")
         remote_client.exec_command(
@@ -396,7 +442,7 @@ class ShareScenarioTest(manager.NetworkScenarioTest):
         remote_client.exec_command(
             "sudo crudini --set ceph.conf client \"mon host\" {}"
             .format(locations[0].split(':/')[0]))
-        return access
+        return access_rule
 
     def wait_for_active_instance(self, instance_id):
         waiters.wait_for_server_status(
@@ -406,8 +452,7 @@ class ShareScenarioTest(manager.NetworkScenarioTest):
 
     def get_share_type(self):
         if CONF.share.default_share_type_name:
-            return self.shares_client.get_share_type(
-                CONF.share.default_share_type_name)['share_type']
+            return self.shares_client.get_default_share_type()['share_type']
         return self._create_share_type(
             data_utils.rand_name("share_type"),
             extra_specs={
@@ -634,14 +679,7 @@ class ShareScenarioTest(manager.NetworkScenarioTest):
         LOG.info('Creating Glance image using the downloaded image file')
         return self._image_create('centos', 'bare', imagepath, 'qcow2')
 
-    def get_share_export_location_for_mount(self, share):
-        exports = self.get_user_export_locations(
-            share=share,
-            error_on_invalid_ip_version=True)
-        return exports[0]
-
-    def get_user_export_locations(self, share=None, snapshot=None,
-                                  error_on_invalid_ip_version=False):
+    def get_user_export_locations(self, share=None, snapshot=None):
         locations = None
         if share:
             locations = self.get_share_export_locations(share)
@@ -651,19 +689,18 @@ class ShareScenarioTest(manager.NetworkScenarioTest):
         self.assertNotEmpty(locations)
         if self.protocol != 'cephfs':
             locations = self._get_export_locations_according_to_ip_version(
-                locations, error_on_invalid_ip_version)
+                locations)
             self.assertNotEmpty(locations)
 
         return locations
 
-    def _get_export_locations_according_to_ip_version(
-            self, all_locations, error_on_invalid_ip_version):
+    def _get_export_locations_according_to_ip_version(self, all_locations):
         locations = [
             x for x in all_locations
             if self.get_ip_and_version_from_export_location(
                 x)[1] == self.ip_version]
 
-        if len(locations) == 0 and not error_on_invalid_ip_version:
+        if len(locations) == 0:
             message = ("Configured backend does not support "
                        "ip_version %s" % self.ip_version)
             raise self.skipException(message)
@@ -686,10 +723,11 @@ class ShareScenarioTest(manager.NetworkScenarioTest):
 
 class BaseShareCEPHFSTest(ShareScenarioTest):
 
-    def allow_access(self, access_level='rw', **kwargs):
+    def allow_access(self, access_level='rw', access_rule=None, **kwargs):
         return self.provide_access_to_client_identified_by_cephx(
             remote_client=kwargs['remote_client'],
-            locations=kwargs['locations'], access_level=access_level)
+            locations=kwargs['locations'], access_level=access_level,
+            access_rule=access_rule)
 
     def _fuse_client(self, mountpoint, remote_client, target_dir, access_to):
         remote_client.exec_command(
